'use strict';
const Gulp = require('gulp');
const RunSequence = require('run-sequence');
const BrowserSync = require('browser-sync');
const Karma = require('karma');

const BuildUtil = require('../util/build');
const Browserify = require('./browserify');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;


Gulp.task('dev', function (callback) {
    TasksUtils.isDev = true;
    RunSequence('build', 'dev-watch', /*'dev-unit-test',*/ callback);
});

Gulp.task('dev-watch', ['dev-watchify', 'dev-browserSync'], function () {
    if (Config.build.lintJs) {
        Gulp.watch(Config.build.lintJs.src, Config.build.lintJs.srcOpts, ['build-lint-js']);
    }

    if (Config.build.sass) {
        Gulp.watch(Config.build.sass.src, ['build-sass']);
    }


    const imageBundles = TasksUtils.arrayOrEmpty(Config.build.images);
    if (imageBundles) {
        imageBundles.forEach((bundle) => {
            Gulp.watch(bundle.src, bundle.srcOpts, () => {
                BuildUtil.buildImages(bundle);
            });
        });
    }

    const copyBundles = TasksUtils.arrayOrEmpty(Config.build.copy);
    if (copyBundles) {
        copyBundles.forEach((bundle) => {
            Gulp.watch(bundle.src, bundle.srcOpts, () => {
                BuildUtil.buildCopy(bundle);
            });
        });
    }

    // Watchify will watch and recompile our JS, so no need to gulp.watch it
    Gulp.watch(['./src/docs/**/*'], ['build-static-site']);
});

Gulp.task('dev-watchify', function () {
    return Browserify.browserifyTask();
});

Gulp.task('dev-browserSync', function () {
    BrowserSync(Config.dev.browserSync);
});

Gulp.task('dev-unit-test', function (done) {
    if (!Config.dev.unitTest) return done();

    Karma.server.start(Config.dev.unitTest.settings, function (exitCode) {
        if (exitCode === 0) {
            done();
        } else {
            done(new Error('Karma exited with code ' + exitCode));
        }
    });
});