﻿'use strict';
const _ = require('lodash');
const TasksUtils = require('./utils');
const GulpUtil = require('gulp-util');
const debug = require('debug')('tp-static-website:static-assets');
const Config = TasksUtils.gulpConfig;


exports = module.exports = {};

// type {string} - 'css' | 'js' | 'images'
const manifestCache = {};
exports.getRevManifest = (type) => {
    // return {} if there should not be any rev manifest
    switch (type) {
        case 'css':
            if (!Config.compile.css) return {};
            break;

        case 'js':
            if (!Config.compile.js) return {};
            break;

        case 'images':
            if (!TasksUtils.arrayOrEmpty(Config.compile.images)) return {};
            break;
    }

    if (manifestCache[type]) return manifestCache[type];
    const manifest = TasksUtils.readJson(`compile/rev/${type}-manifest.json`);
    manifestCache[type] = manifest;
    return manifest;
};

exports.filenameWithRev = function (url) {
    const combinedManifest = _.assign({},
        exports.getRevManifest('css'),
        exports.getRevManifest('js'),
        exports.getRevManifest('images')
    );

    const revvedUrl = combinedManifest[url];

    debug(`filenameWithRev: ${GulpUtil.colors.cyan(url)} => ${GulpUtil.colors.cyan(revvedUrl)}`);

    if (!revvedUrl) {
        GulpUtil.log(`Could not find path with rev for image ${GulpUtil.colors.cyan(url)} in manifest`);
        return url;
    }

    return revvedUrl;
};

exports.cssUrlWithRev = function (url) {
    const imagesManifest = exports.getRevManifest('images');
    let revvedUrl = url;

    _.forEach(imagesManifest, (revvedImagePath, imagePath) => {
        if (_.endsWith(url, imagePath)) {
            revvedUrl = url.slice(0, url.length - imagePath.length) + revvedImagePath;
            return false;
        }
    });

    if (url !== revvedUrl) {
        debug(`cssUrlWithRev: ${GulpUtil.colors.cyan(url)} => ${GulpUtil.colors.cyan(revvedUrl)}`);
    } else {
        debug(`cssUrlWithRev: ${GulpUtil.colors.cyan(url)} => unchanged`);
    }

    return revvedUrl;
};
